@@ -1,7 +1,6 @@
 """This provides bcfg2 support for yum."""
 __revision__ = '$Revision$'
 
-import ConfigParser
 import copy
 import os.path
 import sys
@@ -14,6 +13,8 @@ import yum.misc
 import rpmUtils.arch
 import Bcfg2.Client.XML
 import Bcfg2.Client.Tools
+# Compatibility import
+from Bcfg2.Bcfg2Py3k import ConfigParser
 
 # Fix for python2.3
 try:
@@ -21,6 +22,7 @@ try:
 except NameError:
     from sets import Set as set
 
+
 def build_yname(pkgname, inst):
     """Build yum appropriate package name."""
     d = {}
@@ -39,6 +41,7 @@ def build_yname(pkgname, inst):
             d['arch'] = inst.get('arch')
     return d
 
+
 def short_yname(nevra):
     d = nevra.copy()
     if 'version' in d:
@@ -49,17 +52,19 @@ def short_yname(nevra):
         del d['release']
     return d
 
+
 def nevraString(p):
     if isinstance(p, yum.packages.PackageObject):
         return str(p)
     else:
         ret = ""
-        for i, j in [('epoch','%s:'), ('name','%s'), ('version','-%s'), 
-                     ('release','-%s'), ('arch','.%s')]:
+        for i, j in [('epoch', '%s:'), ('name', '%s'), ('version', '-%s'),
+                     ('release', '-%s'), ('arch', '.%s')]:
             if i in p:
                 ret = "%s%s" % (ret, j % p[i])
         return ret
 
+
 class Parser(ConfigParser.ConfigParser):
 
     def get(self, section, option, default):
@@ -73,6 +78,7 @@ class Parser(ConfigParser.ConfigParser):
         except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
             return default
 
+
 class RPMDisplay(yum.rpmtrans.RPMBaseCallback):
     """We subclass the default RPM transaction callback so that we
        can control Yum's verbosity and pipe it through the right logger."""
@@ -83,11 +89,11 @@ class RPMDisplay(yum.rpmtrans.RPMBaseCallback):
         self.state = None
         self.package = None
 
-    def event(self, package, action, te_current, te_total, 
+    def event(self, package, action, te_current, te_total,
               ts_current, ts_total):
-        """ 
+        """
         @param package: A yum package object or simple string of a package name
-        @param action: A yum.constant transaction set state or in the obscure 
+        @param action: A yum.constant transaction set state or in the obscure
                        rpm repackage case it could be the string 'repackaging'
         @param te_current: Current number of bytes processed in the transaction
                            element being processed
@@ -114,6 +120,7 @@ class RPMDisplay(yum.rpmtrans.RPMBaseCallback):
         """Deal with error reporting."""
         self.logger.error(msg)
 
+
 class YumDisplay(yum.callbacks.ProcessTransBaseCallback):
     """Class to handle display of what step we are in the Yum transaction
        such as downloading packages, etc."""
@@ -136,15 +143,29 @@ class YUMng(Bcfg2.Client.Tools.PkgTool):
                'Path': ['type']}
     __ireq__ = {'Package': ['name']}
 
-    conflicts = ['RPMng']
+    conflicts = ['YUM24', 'RPMng']
 
     def __init__(self, logger, setup, config):
         self.yb = yum.YumBase()
+
+        if setup['debug']:
+            debuglevel = 3
+        elif setup['verbose']:
+            debuglevel = 2
+        else:
+            debuglevel = 1
+
+        try:
+            self.yb.preconf.debuglevel = debuglevel
+        except AttributeError:
+            self.yb._getConfig(self.yb.config_file_path,
+                               debuglevel=debuglevel)
+
         Bcfg2.Client.Tools.PkgTool.__init__(self, logger, setup, config)
-        self.ignores = [ entry.get('name') for struct in config \
-                         for entry in struct \
-                         if entry.tag == 'Path' and \
-                         entry.get('type') == 'ignore' ]
+        self.ignores = [entry.get('name') for struct in config \
+                        for entry in struct \
+                        if entry.tag == 'Path' and \
+                        entry.get('type') == 'ignore']
         self.instance_status = {}
         self.extra_instances = []
         self.modlists = {}
@@ -152,7 +173,7 @@ class YUMng(Bcfg2.Client.Tools.PkgTool):
         self.__important__ = self.__important__ + \
                              [entry.get('name') for struct in config \
                               for entry in struct \
-                              if entry.tag in ['Path', 'ConfigFile'] and \
+                              if entry.tag == 'Path' and \
                               (entry.get('name').startswith('/etc/yum.d') \
                               or entry.get('name').startswith('/etc/yum.repos.d')) \
                               or entry.get('name') == '/etc/yum.conf']
@@ -162,13 +183,15 @@ class YUMng(Bcfg2.Client.Tools.PkgTool):
             self.yb.doConfigSetup()
             self.yb.doTsSetup()
             self.yb.doRpmDBSetup()
-        except yum.Errors.RepoError, e:
+        except yum.Errors.RepoError:
+            e = sys.exc_info()[1]
             self.logger.error("YUMng Repository error: %s" % e)
             raise Bcfg2.Client.Tools.toolInstantiationError
-        except yum.Errors.YumBaseError, e:
+        except Exception:
+            e = sys.exc_info()[1]
             self.logger.error("YUMng error: %s" % e)
             raise Bcfg2.Client.Tools.toolInstantiationError
- 
+
         yup = self.yb.doPackageLists(pkgnarrow='updates')
         if hasattr(self.yb.rpmdb, 'pkglist'):
             yinst = self.yb.rpmdb.pkglist
@@ -201,7 +224,7 @@ class YUMng(Bcfg2.Client.Tools.PkgTool):
                 in truth
         self.doInstall = CP.get(self.name, "installed_action",
                 "install").lower() == "install"
-        self.doUpgrade = CP.get(self.name, 
+        self.doUpgrade = CP.get(self.name,
                 "version_fail_action", "upgrade").lower() == "upgrade"
         self.doReinst = CP.get(self.name, "verify_fail_action",
                 "reinstall").lower() == "reinstall"
@@ -257,7 +280,7 @@ class YUMng(Bcfg2.Client.Tools.PkgTool):
             instance = Bcfg2.Client.XML.SubElement(entry, 'Package')
             for attrib in list(entry.attrib.keys()):
                 instance.attrib[attrib] = entry.attrib[attrib]
-            instances = [ instance ]
+            instances = [instance]
 
         return instances
 
@@ -290,18 +313,20 @@ class YUMng(Bcfg2.Client.Tools.PkgTool):
         else:
             results = verify(po)
             self.verifyCache[key] = results
-        if not rpmUtils.arch.isMultiLibArch(): return results
+        if not rpmUtils.arch.isMultiLibArch():
+            return results
 
         # Okay deal with a buggy yum multilib and verify
         packages = self.yb.rpmdb.searchNevra(name=po.name, epoch=po.epoch,
-                ver=po.version, rel=po.release) # find all arches of pkg
-        if len(packages) == 1: 
+                ver=po.version, rel=po.release)  # find all arches of pkg
+        if len(packages) == 1:
             return results      # No mathcing multilib packages
 
         files = set(po.returnFileEntries())   # Will be the list of common fns
         common = {}
         for p in packages:
-            if p != po: files = files & set(p.returnFileEntries())
+            if p != po:
+                files = files & set(p.returnFileEntries())
         for p in packages:
             k = (p.name, p.epoch, p.version, p.release, p.arch)
             self.logger.debug("Multilib Verify: comparing %s to %s" \
@@ -312,21 +337,22 @@ class YUMng(Bcfg2.Client.Tools.PkgTool):
                 v = verify(p)
                 self.verifyCache[k] = v
 
-            for fn, probs in v.items():
+            for fn, probs in list(v.items()):
                 # file problems must exist in ALL multilib packages to be real
                 if fn in files:
                     common[fn] = common.get(fn, 0) + 1
 
         flag = len(packages) - 1
-        for fn, i in common.items():
+        for fn, i in list(common.items()):
             if i == flag:
                 # this fn had verify problems in all but one of the multilib
                 # packages.  That means its correct in the package that's
                 # "on top."  Therefore, this is a fake verify problem.
-                if fn in results: del results[fn]
+                if fn in results:
+                    del results[fn]
 
         return results
-        
+
     def RefreshPackages(self):
         """
             Creates self.installed{} which is a dict of installed packages.
@@ -415,12 +441,13 @@ class YUMng(Bcfg2.Client.Tools.PkgTool):
                 virtPkg = True
                 self.logger.info("%s appears to be provided by:" \
                                  % entry.get('name'))
-                for p in POs: self.logger.info("  %s" % p)
+                for p in POs:
+                    self.logger.info("  %s" % p)
 
         for inst in instances:
             nevra = build_yname(entry.get('name'), inst)
             snevra = short_yname(nevra)
-            if nevra in packageCache: 
+            if nevra in packageCache:
                 continue   # Ignore duplicate instances
             else:
                 packageCache.append(nevra)
@@ -438,20 +465,26 @@ class YUMng(Bcfg2.Client.Tools.PkgTool):
             verify_flags = inst.get('verify_flags', self.verifyFlags)
             verify_flags = verify_flags.lower().replace(' ', ',').split(',')
 
-            if len(POs) == 0:
-                # Package not installed
+            if 'arch' in nevra:
+                # If arch is specified use it to select the package
+                _POs = [ p for p in POs if p.arch == nevra['arch'] ]
+            else:
+                _POs = POs
+            if len(_POs) == 0:
+                # Package (name, arch) not installed
                 self.logger.debug("  %s is not installed" % nevraString(nevra))
                 stat['installed'] = False
                 package_fail = True
                 qtext_versions.append("I(%s)" % nevra)
                 continue
 
-            if not pkg_checks: continue
+            if not pkg_checks:
+                continue
 
             # Check EVR
             if virtPkg:
                 self.logger.debug("  Not checking version for virtual package")
-                _POs = [ po for po in POs ]  # Make a copy
+                _POs = [po for po in POs]  # Make a copy
             elif entry.get('name') == 'gpg-pubkey':
                 if 'version' not in nevra:
                     m = "Skipping verify: gpg-pubkey without an RPM version."
@@ -469,12 +502,16 @@ class YUMng(Bcfg2.Client.Tools.PkgTool):
                 package_fail = True
                 stat['version_fail'] = True
                 # Just chose the first pkg for the error message
-                self.logger.info("  Wrong version installed.  "\
-                        "Want %s, but have %s" % (nevraString(nevra), 
-                                                  nevraString(POs[0])))
+                self.logger.info("  %s: Wrong version installed.  "
+                                 "Want %s, but have %s" % (entry.get("name"),
+                                                           nevraString(nevra),
+                                                           nevraString(POs[0])))
                 qtext_versions.append("U(%s)" % str(POs[0]))
                 continue
 
+            if self.setup.get('quick', False):
+                # Passed -q on the command line
+                continue
             if not (pkg_verify and \
                     inst.get('pkg_verify', 'true').lower() == 'true'):
                 continue
@@ -492,7 +529,8 @@ class YUMng(Bcfg2.Client.Tools.PkgTool):
 
             try:
                 vResult = self._verifyHelper(_POs[0])
-            except Exception, e:
+            except Exception:
+                e = sys.exc_info()[1]
                 # Unknown Yum exception
                 self.logger.warning("  Verify Exception: %s" % str(e))
                 package_fail = True
@@ -502,8 +540,8 @@ class YUMng(Bcfg2.Client.Tools.PkgTool):
             ignores = [ig.get('name') for ig in entry.findall('Ignore')] + \
                       [ig.get('name') for ig in inst.findall('Ignore')] + \
                       self.ignores
-            for fn, probs in vResult.items():
-                if fn in modlist: 
+            for fn, probs in list(vResult.items()):
+                if fn in modlist:
                     self.logger.debug("  %s in modlist, skipping" % fn)
                     continue
                 if fn in ignores:
@@ -519,18 +557,22 @@ class YUMng(Bcfg2.Client.Tools.PkgTool):
                         tmp.append((p.type, p.message))
                 if tmp != []:
                     stat['verify'][fn] = tmp
-            
+
             if stat['verify'] != {}:
                 stat['verify_fail'] = True
                 package_fail = True
+                self.logger.debug("It is suggested that you either manage "
+                                  "these files, revert the changes, or ignore "
+                                  "false failures:")
                 self.logger.debug("  Verify Problems:")
-                for fn, probs in stat['verify'].items():
+                for fn, probs in list(stat['verify'].items()):
                     self.logger.debug("    %s" % fn)
-                    for p in probs: self.logger.debug("      %s: %s" % p)
+                    for p in probs:
+                        self.logger.debug("      %s: %s" % p)
 
         if len(POs) > 0:
             # Is this an install only package?  We just look at the first one
-            provides = set([ p[0] for p in POs[0].provides ] + [ POs[0].name ])
+            provides = set([p[0] for p in POs[0].provides] + [POs[0].name])
             install_only = len(set(self.installOnlyPkgs) & provides) > 0
         else:
             install_only = False
@@ -550,7 +592,6 @@ class YUMng(Bcfg2.Client.Tools.PkgTool):
 
         return not package_fail
 
-
     def FindExtraInstances(self, entry, POs):
         """
             Check for installed instances that are not in the config.
@@ -558,12 +599,13 @@ class YUMng(Bcfg2.Client.Tools.PkgTool):
             are no Instances to remove.
 
         """
-        if len(POs) == 0: return None
+        if len(POs) == 0:
+            return None
         name = entry.get('name')
-        extra_entry = Bcfg2.Client.XML.Element('Package', name=name, 
+        extra_entry = Bcfg2.Client.XML.Element('Package', name=name,
                                                type=self.pkgtype)
         instances = self._buildInstances(entry)
-        _POs = [ p for p in POs ] # Shallow copy
+        _POs = [p for p in POs]  # Shallow copy
 
         # Algorythm is sensitive to duplicates, check for them
         checked = []
@@ -574,7 +616,7 @@ class YUMng(Bcfg2.Client.Tools.PkgTool):
             flag = True
             if len(pkgs) > 0:
                 if pkgs[0] in checked:
-                    continue # We've already taken care of this Instance
+                    continue  # We've already taken care of this Instance
                 else:
                     checked.append(pkgs[0])
                 _POs.remove(pkgs[0])
@@ -585,26 +627,27 @@ class YUMng(Bcfg2.Client.Tools.PkgTool):
                     epoch=p.epoch, name=p.name, version=p.version,
                     release=p.release, arch=p.arch)
 
-        if _POs == []: 
+        if _POs == []:
             return None
         else:
             return extra_entry
 
     def FindExtraPackages(self):
         """Find extra packages."""
-        packages = [ e.get('name') for e in self.getSupportedEntries() ]
+        packages = [e.get('name') for e in self.getSupportedEntries()]
         extras = []
 
-        for p in self.installed.keys():
+        for p in list(self.installed.keys()):
             if p not in packages:
-                entry = Bcfg2.Client.XML.Element('Package', name=p, 
+                entry = Bcfg2.Client.XML.Element('Package', name=p,
                                                  type=self.pkgtype)
                 for i in self.installed[p]:
-                    inst = Bcfg2.Client.XML.SubElement(entry, 'Instance', \
-                            epoch = i['epoch'],
-                            version = i['version'],
-                            release = i['release'],
-                            arch = i['arch'])
+                    inst = Bcfg2.Client.XML.SubElement(entry,
+                                                       'Instance',
+                                                       epoch=i['epoch'],
+                                                       version=i['version'],
+                                                       release=i['release'],
+                                                       arch=i['arch'])
 
                 extras.append(entry)
 
@@ -628,16 +671,16 @@ class YUMng(Bcfg2.Client.Tools.PkgTool):
         rel = yum.misc.keyIdToRPMVer(gpg['timestamp'])
         if not (ver == inst.get('version') and rel == inst.get('release')):
             self.logger.info("GPG key file %s does not match gpg-pubkey-%s-%s"\
-                             % (key_file, inst.get('version'), 
+                             % (key_file, inst.get('version'),
                                 inst.get('release')))
             return False
 
-        if not yum.misc.keyInstalled(ts, gpg['keyid'], 
+        if not yum.misc.keyInstalled(ts, gpg['keyid'],
                                      gpg['timestamp']) == 0:
             result = ts.pgpImportPubkey(yum.misc.procgpgkey(rawkey))
         else:
             self.logger.debug("gpg-pubkey-%s-%s already installed"\
-                              % (inst.get('version'), 
+                              % (inst.get('version'),
                                  inst.get('release')))
             return True
 
@@ -653,38 +696,58 @@ class YUMng(Bcfg2.Client.Tools.PkgTool):
             return True
 
     def _runYumTransaction(self):
+        def cleanup():
+            self.yb.closeRpmDB()
+            self.RefreshPackages()
+
         rDisplay = RPMDisplay(self.logger)
         yDisplay = YumDisplay(self.logger)
         # Run the Yum Transaction
-        rescode, restring = self.yb.buildTransaction()
+        try:
+            rescode, restring = self.yb.buildTransaction()
+        except yum.Errors.YumBaseError:
+            e = sys.exc_info()[1]
+            self.logger.error("Yum transaction error: %s" % str(e))
+            cleanup()
+            return
+
         self.logger.debug("Initial Yum buildTransaction() run said:")
         self.logger.debug("   resultcode: %s, msgs: %s" \
                           % (rescode, restring))
 
         if rescode != 1:
             # Transaction built successfully, run it
-            self.yb.processTransaction(callback=yDisplay,
-                                       rpmDisplay=rDisplay)
-            self.logger.info("Single Pass for Install Succeeded")
+            try:
+                self.yb.processTransaction(callback=yDisplay,
+                                           rpmDisplay=rDisplay)
+                self.logger.info("Single Pass for Install Succeeded")
+            except yum.Errors.YumBaseError:
+                e = sys.exc_info()[1]
+                self.logger.error("Yum transaction error: %s" % str(e))
+                cleanup()
+                return
         else:
             # The yum command failed.  No packages installed.
             # Try installing instances individually.
             self.logger.error("Single Pass Install of Packages Failed")
             skipBroken = self.yb.conf.skip_broken
             self.yb.conf.skip_broken = True
-            rescode, restring = self.yb.buildTransaction()
-            if rescode != 1:
-                self.yb.processTransaction(callback=yDisplay,
-                                           rpmDisplay=rDisplay)
-                self.logger.debug(
-                    "Second pass install did not install all packages")
-            else:
-                self.logger.error("Second pass yum install failed.")
-                self.logger.debug("   %s" % restring)
+            try:
+                rescode, restring = self.yb.buildTransaction()
+                if rescode != 1:
+                    self.yb.processTransaction(callback=yDisplay,
+                                               rpmDisplay=rDisplay)
+                    self.logger.debug(
+                        "Second pass install did not install all packages")
+                else:
+                    self.logger.error("Second pass yum install failed.")
+                    self.logger.debug("   %s" % restring)
+            except yum.Errors.YumBaseError, e:
+                self.logger.error("Yum transaction error: %s" % str(e))
+
             self.yb.conf.skip_broken = skipBroken
 
-        self.yb.closeRpmDB()
-        self.RefreshPackages()
+        cleanup()
 
     def Install(self, packages, states):
         """
@@ -784,30 +847,38 @@ class YUMng(Bcfg2.Client.Tools.PkgTool):
 
             for inst in install_pkgs:
                 pkg_arg = self.instance_status[inst].get('pkg').get('name')
+                self.logger.debug("Installing %s" % pkg_arg)
                 try:
                     self.yb.install(**build_yname(pkg_arg, inst))
-                except yum.Errors.YumBaseError, yume:
-                    self.logger.error("Error installing some packages: %s" % yume)
+                except yum.Errors.YumBaseError:
+                    yume = sys.exc_info()[1]
+                    self.logger.error("Error installing package %s: %s" %
+                                      (pkg_arg, yume))
 
         if len(upgrade_pkgs) > 0:
             self.logger.info("Attempting to upgrade packages")
 
             for inst in upgrade_pkgs:
                 pkg_arg = self.instance_status[inst].get('pkg').get('name')
+                self.logger.debug("Upgrading %s" % pkg_arg)
                 try:
                     self.yb.update(**build_yname(pkg_arg, inst))
-                except yum.Errors.YumBaseError, yume:
-                    self.logger.error("Error upgrading some packages: %s" % yume)
+                except yum.Errors.YumBaseError:
+                    yume = sys.exc_info()[1]
+                    self.logger.error("Error upgrading package %s: %s" %
+                                      (pkg_arg, yume))
 
         if len(reinstall_pkgs) > 0:
             self.logger.info("Attempting to reinstall packages")
             for inst in reinstall_pkgs:
                 pkg_arg = self.instance_status[inst].get('pkg').get('name')
+                self.logger.debug("Reinstalling %s" % pkg_arg)
                 try:
                     self.yb.reinstall(**build_yname(pkg_arg, inst))
-                except yum.Errors.YumBaseError, yume:
-                    self.logger.error("Error upgrading some packages: %s" \
-                            % yume)
+                except yum.Errors.YumBaseError:
+                    yume = sys.exc_info()[1]
+                    self.logger.error("Error reinstalling package %s: %s" %
+                                      (pkg_arg, yume))
 
         self._runYumTransaction()
 
