@@ -11,9 +11,6 @@ load_config -- read configuration files
 __revision__ = '$Revision: $'
 
 
-from xmlrpclib import _Method
-
-import httplib
 import logging
 import re
 import socket
@@ -25,7 +22,7 @@ import socket
 try:
     import ssl
     SSL_LIB = 'py26_ssl'
-except ImportError, e:
+except ImportError:
     from M2Crypto import SSL
     import M2Crypto.SSL.Checker
     SSL_LIB = 'm2crypto'
@@ -33,22 +30,47 @@ except ImportError, e:
 
 import sys
 import time
-import urlparse
-import xmlrpclib
+
+# Compatibility imports
+from Bcfg2.Bcfg2Py3k import httplib, xmlrpclib, urlparse
 
 version = sys.version_info[:2]
-has_py23 = map(int, version) >= [2, 3]
-has_py26 = map(int, version) >= [2, 6]
+has_py23 = version >= (2, 3)
+has_py26 = version >= (2, 6)
 
-__all__ = ["ComponentProxy", "RetryMethod", "SSLHTTPConnection", "XMLRPCTransport"]
+__all__ = ["ComponentProxy",
+           "RetryMethod",
+           "SSLHTTPConnection",
+           "XMLRPCTransport"]
 
 
+class ProxyError(Exception):
+    """ ProxyError provides a consistent reporting interface to
+    the various xmlrpclib errors that might arise (mainly
+    ProtocolError and Fault) """
+    def __init__(self, err):
+        if isinstance(err, xmlrpclib.ProtocolError):
+            # cut out the password in the URL
+            url = re.sub(r'([^:]+):(.*?)@([^@]+:\d+/)', r'\1:******@\3',
+                         err.url)
+            self.message = "XML-RPC Protocol Error for %s: %s (%s)" % \
+                           (url, err.errmsg, err.errcode)
+        elif isinstance(err, xmlrpclib.Fault):
+            self.message = "XML-RPC Fault: %s (%s)" % (err.faultString,
+                                                       err.faultCode)
+        else:
+            self.message = str(err)
+        self.args = (self.message, )
+
 class CertificateError(Exception):
     def __init__(self, commonName):
         self.commonName = commonName
+    def __str__(self):
+        return ("Got unallowed commonName %s from server"
+                % self.commonName)
 
 
-class RetryMethod(_Method):
+class RetryMethod(xmlrpclib._Method):
     """Method with error handling and retries built in."""
     log = logging.getLogger('xmlrpc')
     max_retries = 4
@@ -56,21 +78,27 @@ class RetryMethod(_Method):
     def __call__(self, *args):
         for retry in range(self.max_retries):
             try:
-                return _Method.__call__(self, *args)
-            except xmlrpclib.ProtocolError, err:
+                return xmlrpclib._Method.__call__(self, *args)
+            except xmlrpclib.ProtocolError:
+                err = sys.exc_info()[1]
                 self.log.error("Server failure: Protocol Error: %s %s" % \
                               (err.errcode, err.errmsg))
                 raise xmlrpclib.Fault(20, "Server Failure")
             except xmlrpclib.Fault:
                 raise
-            except socket.error, err:
+            except socket.error:
+                err = sys.exc_info()[1]
                 if hasattr(err, 'errno') and err.errno == 336265218:
                     self.log.error("SSL Key error")
                     break
+                if hasattr(err, 'errno') and err.errno == 185090050:
+                    self.log.error("SSL CA error")
+                    break
                 if retry == 3:
                     self.log.error("Server failure: %s" % err)
                     raise xmlrpclib.Fault(20, err)
-            except CertificateError, ce:
+            except CertificateError:
+                ce = sys.exc_info()[1]
                 self.log.error("Got unallowed commonName %s from server" \
                                % ce.commonName)
                 break
@@ -84,11 +112,13 @@ class RetryMethod(_Method):
         raise xmlrpclib.Fault(20, "Server Failure")
 
 # sorry jon
-xmlrpclib._Method = RetryMethod
+_Method = RetryMethod
 
 
 class SSLHTTPConnection(httplib.HTTPConnection):
-    """Extension of HTTPConnection that implements SSL and related behaviors."""
+    """Extension of HTTPConnection that
+    implements SSL and related behaviors.
+    """
 
     logger = logging.getLogger('Bcfg2.Proxy.SSLHTTPConnection')
 
@@ -157,7 +187,7 @@ class SSLHTTPConnection(httplib.HTTPConnection):
         elif SSL_LIB == 'm2crypto':
             self._connect_m2crypto()
         else:
-            raise Exception, "No SSL module support"
+            raise Exception("No SSL module support")
 
     def _connect_py26ssl(self):
         """Initiates a connection using the ssl module."""
@@ -168,7 +198,7 @@ class SSLHTTPConnection(httplib.HTTPConnection):
             ssl_protocol_ver = ssl.PROTOCOL_TLSv1
         else:
             self.logger.error("Unknown protocol %s" % (self.protocol))
-            raise Exception, "unknown protocol %s" % self.protocol
+            raise Exception("unknown protocol %s" % self.protocol)
         if self.ca:
             other_side_required = ssl.CERT_REQUIRED
         else:
@@ -192,7 +222,7 @@ class SSLHTTPConnection(httplib.HTTPConnection):
         if peer_cert and self.scns:
             scn = [x[0][1] for x in peer_cert['subject'] if x[0][0] == 'commonName'][0]
             if scn not in self.scns:
-                raise CertificateError, scn
+                raise CertificateError(scn)
         self.sock.closeSocket = True
 
     def _connect_m2crypto(self):
@@ -204,7 +234,7 @@ class SSLHTTPConnection(httplib.HTTPConnection):
             ctx = SSL.Context('tlsv1')
         else:
             self.logger.error("Unknown protocol %s" % (self.protocol))
-            raise Exception, "unknown protocol %s" % self.protocol
+            raise Exception("unknown protocol %s" % self.protocol)
 
         if self.ca:
             # Use the certificate authority to validate the cert
@@ -237,8 +267,9 @@ class SSLHTTPConnection(httplib.HTTPConnection):
         try:
             self.sock.connect((hostname, self.port))
             # automatically checks cert matches host
-        except M2Crypto.SSL.Checker.WrongHost, wr:
-            raise CertificateError, wr
+        except M2Crypto.SSL.Checker.WrongHost:
+            wr = sys.exc_info()[1]
+            raise CertificateError(wr)
 
 
 class XMLRPCTransport(xmlrpclib.Transport):
@@ -254,8 +285,12 @@ class XMLRPCTransport(xmlrpclib.Transport):
 
     def make_connection(self, host):
         host, self._extra_headers = self.get_host_info(host)[0:2]
-        http = SSLHTTPConnection(host, key=self.key, cert=self.cert, ca=self.ca,
-                                 scns=self.scns, timeout=self.timeout)
+        http = SSLHTTPConnection(host,
+                                 key=self.key,
+                                 cert=self.cert,
+                                 ca=self.ca,
+                                 scns=self.scns,
+                                 timeout=self.timeout)
         https = httplib.HTTP()
         https._setup(http)
         return https
@@ -268,10 +303,25 @@ class XMLRPCTransport(xmlrpclib.Transport):
         self.send_user_agent(h)
         self.send_content(h, request_body)
 
-        errcode, errmsg, headers = h.getreply()
+        if SSL_LIB == 'py26_ssl':
+            catch = ssl.SSLError
+        elif SSL_LIB == 'm2crypto':
+            catch = SSL.SSLError
 
+        try:
+            errcode, errmsg, headers = h.getreply()
+        except catch:
+            err = sys.exc_info()[1]
+            raise ProxyError(xmlrpclib.ProtocolError(host + handler,
+                                                     408,
+                                                     str(err),
+                                                     self._extra_headers))
+        
         if errcode != 200:
-            raise xmlrpclib.ProtocolError(host + handler, errcode, errmsg, headers)
+            raise ProxyError(xmlrpclib.ProtocolError(host + handler,
+                                                     errcode,
+                                                     errmsg,
+                                                     headers))
 
         self.verbose = verbose
         msglen = int(headers.dict['content-length'])
@@ -290,7 +340,7 @@ class XMLRPCTransport(xmlrpclib.Transport):
             if not response:
                 break
             if self.verbose:
-                print "body:", repr(response), len(response)
+                print("body:", repr(response), len(response))
             p.feed(response)
 
         fd.close()
@@ -313,10 +363,10 @@ def ComponentProxy(url, user=None, password=None,
     """
 
     if user and password:
-        method, path = urlparse.urlparse(url)[:2]
+        method, path = urlparse(url)[:2]
         newurl = "%s://%s:%s@%s" % (method, user, password, path)
     else:
         newurl = url
     ssl_trans = XMLRPCTransport(key, cert, ca,
-                                allowedServerCNs, timeout=timeout)
+                                allowedServerCNs, timeout=float(timeout))
     return xmlrpclib.ServerProxy(newurl, allow_none=True, transport=ssl_trans)
