@@ -1,33 +1,41 @@
 import getopt
 from subprocess import Popen, PIPE
+import sys
 import pipes
 import Bcfg2.Server.Admin
 
+
 class Viz(Bcfg2.Server.Admin.MetadataCore):
     __shorthelp__ = "Produce graphviz diagrams of metadata structures"
     __longhelp__ = (__shorthelp__ + "\n\nbcfg2-admin viz [--includehosts] "
                                     "[--includebundles] [--includekey] "
-                                    "[-o output.png] [--raw]")
+                                    "[--only-client clientname] "
+                                    "[-o output.<ext>]\n")
     __usage__ = ("bcfg2-admin viz [options]\n\n"
-                 "     %-25s%s\n"
-                 "     %-25s%s\n"
-                 "     %-25s%s\n"
-                 "     %-25s%s\n" %
+                 "     %-32s%s\n"
+                 "     %-32s%s\n"
+                 "     %-32s%s\n"
+                 "     %-32s%s\n"
+                 "     %-32s%s\n" %
                 ("-H, --includehosts",
                  "include hosts in the viz output",
                  "-b, --includebundles",
                  "include bundles in the viz output",
                  "-k, --includekey",
                  "show a key for different digraph shapes",
+                 "-c, --only-client <clientname>",
+                 "show only the groups, bundles for the named client",
                  "-o, --outfile <file>",
                  "write viz output to an output file"))
 
     colors = ['steelblue1', 'chartreuse', 'gold', 'magenta',
               'indianred1', 'limegreen', 'orange1', 'lightblue2',
               'green1', 'blue1', 'yellow1', 'darkturquoise', 'gray66']
+
     plugin_blacklist = ['DBStats', 'Snapshots', 'Cfg', 'Pkgmgr', 'Packages',
                         'Rules', 'Account', 'Decisions', 'Deps', 'Git', 'Svn',
-                        'Fossil', 'Bzr', 'Bundler', 'TGenshi', 'SGenshi', 'Base']
+                        'Fossil', 'Bzr', 'Bundler', 'TGenshi', 'SGenshi',
+                        'Base']
 
     def __init__(self, cfile):
 
@@ -39,17 +47,19 @@ class Viz(Bcfg2.Server.Admin.MetadataCore):
         Bcfg2.Server.Admin.MetadataCore.__call__(self, args)
         # First get options to the 'viz' subcommand
         try:
-            opts, args = getopt.getopt(args, 'Hbko:',
+            opts, args = getopt.getopt(args, 'Hbkc:o:',
                                        ['includehosts', 'includebundles',
-                                        'includekey', 'outfile='])
-        except getopt.GetoptError, msg:
-            print msg
+                                        'includekey', 'only-client=', 'outfile='])
+        except getopt.GetoptError:
+            msg = sys.exc_info()[1]
+            print(msg)
+            print(self.__longhelp__)
+            raise SystemExit(1)
 
-        #FIXME: is this for --raw?
-        #rset = False
         hset = False
         bset = False
         kset = False
+        only_client = None
         outputfile = False
         for opt, arg in opts:
             if opt in ("-H", "--includehosts"):
@@ -58,17 +68,19 @@ class Viz(Bcfg2.Server.Admin.MetadataCore):
                 bset = True
             elif opt in ("-k", "--includekey"):
                 kset = True
+            elif opt in ("-c", "--only-client"):
+                only_client = arg
             elif opt in ("-o", "--outfile"):
                 outputfile = arg
 
         data = self.Visualize(self.get_repo_path(), hset, bset,
-                              kset, outputfile)
+                              kset, only_client, outputfile)
         if data:
             print(data)
-        raise SystemExit, 0
+        raise SystemExit(0)
 
     def Visualize(self, repopath, hosts=False,
-                  bundles=False, key=False, output=False):
+                  bundles=False, key=False, only_client=None, output=False):
         """Build visualization of groups file."""
         if output:
             format = output.split('.')[-1]
@@ -93,11 +105,11 @@ class Viz(Bcfg2.Server.Admin.MetadataCore):
         try:
             dotpipe.stdin.write("digraph groups {\n")
         except:
-            print "write to dot process failed. Is graphviz installed?"
+            print("write to dot process failed. Is graphviz installed?")
             raise SystemExit(1)
         dotpipe.stdin.write('\trankdir="LR";\n')
         dotpipe.stdin.write(self.metadata.viz(hosts, bundles,
-                                                key, self.colors))
+                                              key, only_client, self.colors))
         if key:
             dotpipe.stdin.write("\tsubgraph cluster_key {\n")
             dotpipe.stdin.write('''\tstyle="filled";\n''')
