@@ -1,13 +1,16 @@
+import sys
+
 import Bcfg2.Options
 import Bcfg2.Proxy
 import Bcfg2.Server.Admin
 
-import sys
-import xmlrpclib
+# Compatibility import
+from Bcfg2.Bcfg2Py3k import xmlrpclib
+
 
 class Xcmd(Bcfg2.Server.Admin.Mode):
     __shorthelp__ = ("XML-RPC Command Interface")
-    __longhelp__ = (__shorthelp__ + "\n\nbcfg2-admin xcmd command")
+    __longhelp__ = (__shorthelp__ + "\n\nbcfg2-admin xcmd command\n")
     __usage__ = ("bcfg2-admin xcmd <command>")
 
     def __call__(self, args):
@@ -16,18 +19,20 @@ class Xcmd(Bcfg2.Server.Admin.Mode):
             'user': Bcfg2.Options.CLIENT_USER,
             'password': Bcfg2.Options.SERVER_PASSWORD,
             'key': Bcfg2.Options.SERVER_KEY,
-            'certificate'     : Bcfg2.Options.CLIENT_CERT,
-            'ca'              : Bcfg2.Options.CLIENT_CA
+            'certificate': Bcfg2.Options.CLIENT_CERT,
+            'ca': Bcfg2.Options.CLIENT_CA,
+            'timeout': Bcfg2.Options.CLIENT_TIMEOUT,
             }
         setup = Bcfg2.Options.OptionParser(optinfo)
-        setup.parse(sys.argv[2:])
+        setup.parse(args)
         Bcfg2.Proxy.RetryMethod.max_retries = 1
         proxy = Bcfg2.Proxy.ComponentProxy(setup['server'],
                                            setup['user'],
                                            setup['password'],
-                                           key = setup['key'],
-                                           cert = setup['certificate'],
-                                           ca = setup['ca'], timeout=180)
+                                           key=setup['key'],
+                                           cert=setup['certificate'],
+                                           ca=setup['ca'],
+                                           timeout=setup['timeout'])
         if len(setup['args']) == 0:
             print("Usage: xcmd <xmlrpc method> <optional arguments>")
             return
@@ -36,8 +41,9 @@ class Xcmd(Bcfg2.Server.Admin.Mode):
         if len(setup['args']) > 1:
             args = tuple(setup['args'][1:])
         try:
-            data = apply(getattr(proxy, cmd), args)
-        except xmlrpclib.Fault, flt:
+            data = getattr(proxy, cmd)(*args)
+        except xmlrpclib.Fault:
+            flt = sys.exc_info()[1]
             if flt.faultCode == 7:
                 print("Unknown method %s" % cmd)
                 return
@@ -46,4 +52,4 @@ class Xcmd(Bcfg2.Server.Admin.Mode):
             else:
                 raise
         if data != None:
-            print data
+            print(data)
