@@ -3,6 +3,7 @@ __revision__ = '$Revision$'
 
 import Bcfg2.Server.Plugin
 
+
 class Account(Bcfg2.Server.Plugin.Plugin,
               Bcfg2.Server.Plugin.Generator):
     """This module generates account config files,
@@ -21,13 +22,14 @@ class Account(Bcfg2.Server.Plugin.Plugin,
     def __init__(self, core, datastore):
         Bcfg2.Server.Plugin.Plugin.__init__(self, core, datastore)
         Bcfg2.Server.Plugin.Generator.__init__(self)
-        self.Entries = {'ConfigFile':{'/etc/passwd':self.from_yp_cb,
-                                      '/etc/group':self.from_yp_cb,
-                                      '/etc/security/limits.conf':self.gen_limits_cb,
-                                      '/root/.ssh/authorized_keys':self.gen_root_keys_cb,
-                                      '/etc/sudoers':self.gen_sudoers}}
+        self.Entries = {'ConfigFile': {'/etc/passwd': self.from_yp_cb,
+                                       '/etc/group': self.from_yp_cb,
+                                       '/etc/security/limits.conf': self.gen_limits_cb,
+                                       '/root/.ssh/authorized_keys': self.gen_root_keys_cb,
+                                       '/etc/sudoers': self.gen_sudoers}}
         try:
-            self.repository = Bcfg2.Server.Plugin.DirectoryBacked(self.data, self.core.fam)
+            self.repository = Bcfg2.Server.Plugin.DirectoryBacked(self.data,
+                                                                  self.core.fam)
         except:
             self.logger.error("Failed to load repos: %s, %s" % \
                               (self.data, "%s/ssh" % (self.data)))
@@ -38,9 +40,11 @@ class Account(Bcfg2.Server.Plugin.Plugin,
         fname = entry.attrib['name'].split('/')[-1]
         entry.text = self.repository.entries["static.%s" % (fname)].data
         entry.text += self.repository.entries["dyn.%s" % (fname)].data
-        perms = {'owner':'root', 'group':'root', 'perms':'0644'}
+        perms = {'owner': 'root',
+                 'group': 'root',
+                 'perms': '0644'}
         [entry.attrib.__setitem__(key, value) for (key, value) in \
-         perms.iteritems()]
+         list(perms.items())]
 
     def gen_limits_cb(self, entry, metadata):
         """Build limits entries based on current ACLs."""
@@ -50,9 +54,11 @@ class Account(Bcfg2.Server.Plugin.Plugin,
                       self.repository.entries["useraccess"].data.split()]
         users = [user for (user, host) in \
                  useraccess if host == metadata.hostname.split('.')[0]]
-        perms = {'owner':'root', 'group':'root', 'perms':'0600'}
+        perms = {'owner': 'root',
+                 'group': 'root',
+                 'perms': '0600'}
         [entry.attrib.__setitem__(key, value) for (key, value) in \
-         perms.iteritems()]
+         list(perms.items())]
         entry.text += "".join(["%s hard maxlogins 1024\n" % uname for uname in superusers + users])
         if "*" not in users:
             entry.text += "* hard maxlogins 0\n"
@@ -71,9 +77,11 @@ class Account(Bcfg2.Server.Plugin.Plugin,
         entry.text = "".join([rdata["%s.key" % user].data for user \
                               in superusers if \
                               ("%s.key" % user) in rdata])
-        perms = {'owner':'root', 'group':'root', 'perms':'0600'}
+        perms = {'owner': 'root',
+                 'group': 'root',
+                 'perms': '0600'}
         [entry.attrib.__setitem__(key, value) for (key, value) \
-         in perms.iteritems()]
+         in list(perms.items())]
 
     def gen_sudoers(self, entry, metadata):
         """Build root authorized keys file based on current ACLs."""
@@ -88,6 +96,8 @@ class Account(Bcfg2.Server.Plugin.Plugin,
         entry.text = self.repository.entries['static.sudoers'].data
         entry.text += "".join(["%s ALL=(ALL) ALL\n" % uname \
                                for uname in superusers])
-        perms = {'owner':'root', 'group':'root', 'perms':'0440'}
+        perms = {'owner': 'root',
+                 'group': 'root',
+                 'perms': '0440'}
         [entry.attrib.__setitem__(key, value) for (key, value) \
-         in perms.iteritems()]
+         in list(perms.items())]
