@@ -1,4 +1,6 @@
-"""This file stores persistent metadata for the Bcfg2 Configuration Repository."""
+"""
+This file stores persistent metadata for the Bcfg2 Configuration Repository.
+"""
 
 __revision__ = '$Revision$'
 
@@ -8,18 +10,166 @@ import lxml.etree
 import os
 import os.path
 import socket
+import sys
 import time
+
 import Bcfg2.Server.FileMonitor
 import Bcfg2.Server.Plugin
 
+
+def locked(fd):
+    """Aquire a lock on a file"""
+    try:
+        fcntl.lockf(fd, fcntl.LOCK_EX | fcntl.LOCK_NB)
+    except IOError:
+        return True
+    return False
+
+
 class MetadataConsistencyError(Exception):
     """This error gets raised when metadata is internally inconsistent."""
     pass
 
+
 class MetadataRuntimeError(Exception):
-    """This error is raised when the metadata engine is called prior to reading enough data."""
+    """This error is raised when the metadata engine
+    is called prior to reading enough data.
+    """
     pass
 
+
+class XMLMetadataConfig(object):
+    """Handles xml config files and all XInclude statements"""
+    def __init__(self, metadata, watch_clients, basefile):
+        self.metadata = metadata
+        self.basefile = basefile
+        self.should_monitor = watch_clients
+        self.extras = []
+        self.data = None
+        self.basedata = None
+        self.basedir = metadata.data
+        self.logger = metadata.logger
+        self.pseudo_monitor = isinstance(metadata.core.fam,
+                                         Bcfg2.Server.FileMonitor.Pseudo)
+
+    @property
+    def xdata(self):
+        if not self.data:
+            raise MetadataRuntimeError
+        return self.data
+
+    @property
+    def base_xdata(self):
+        if not self.basedata:
+            raise MetadataRuntimeError
+        return self.basedata
+
+    def add_monitor(self, fname):
+        """Add a fam monitor for an included file"""
+        if self.should_monitor:
+            self.metadata.core.fam.AddMonitor("%s/%s" % (self.basedir, fname),
+                                              self.metadata)
+            self.extras.append(fname)
+
+    def load_xml(self):
+        """Load changes from XML"""
+        try:
+            xdata = lxml.etree.parse("%s/%s" % (self.basedir, self.basefile))
+        except lxml.etree.XMLSyntaxError:
+            self.logger.error('Failed to parse %s' % (self.basefile))
+            return
+        self.basedata = copy.deepcopy(xdata)
+        included = [ent.get('href') for ent in \
+                    xdata.findall('./{http://www.w3.org/2001/XInclude}include')]
+        if included:
+            for name in included:
+                if name not in self.extras:
+                    self.add_monitor(name)
+            try:
+                xdata.xinclude()
+            except lxml.etree.XIncludeError:
+                self.logger.error("Failed to process XInclude for file %s" % self.basefile)
+        self.data = xdata
+
+    def write(self):
+        """Write changes to xml back to disk."""
+        self.write_xml("%s/%s" % (self.basedir, self.basefile),
+                       self.basedata)
+
+    def write_xml(self, fname, xmltree):
+        """Write changes to xml back to disk."""
+        tmpfile = "%s.new" % fname
+        try:
+            datafile = open("%s" % tmpfile, 'w')
+        except IOError:
+            e = sys.exc_info()[1]
+            self.logger.error("Failed to write %s: %s" % (tmpfile, e))
+            raise MetadataRuntimeError
+        # prep data
+        dataroot = xmltree.getroot()
+        newcontents = lxml.etree.tostring(dataroot, pretty_print=True)
+
+        fd = datafile.fileno()
+        while locked(fd) == True:
+            pass
+        try:
+            datafile.write(newcontents)
+        except:
+            fcntl.lockf(fd, fcntl.LOCK_UN)
+            self.logger.error("Metadata: Failed to write new xml data to %s" % tmpfile, exc_info=1)
+            os.unlink("%s" % tmpfile)
+            raise MetadataRuntimeError
+        datafile.close()
+
+        # check if clients.xml is a symlink
+        xmlfile = "%s" % fname
+        if os.path.islink(xmlfile):
+            xmlfile = os.readlink(xmlfile)
+
+        try:
+            os.rename("%s" % tmpfile, xmlfile)
+        except:
+            self.logger.error("Metadata: Failed to rename %s" % tmpfile)
+            raise MetadataRuntimeError
+
+    def find_xml_for_xpath(self, xpath):
+        """Find and load xml data containing the xpath query"""
+        if self.pseudo_monitor:
+            # Reload xml if we don't have a real monitor
+            self.load_xml()
+        cli = self.basedata.xpath(xpath)
+        if len(cli) > 0:
+            return {'filename': "%s/%s" % (self.basedir, self.basefile),
+                    'xmltree': self.basedata,
+                    'xquery': cli}
+        else:
+            """Try to find the data in included files"""
+            for included in self.extras:
+                try:
+                    xdata = lxml.etree.parse("%s/%s" % (self.basedir, included))
+                    cli = xdata.xpath(xpath)
+                    if len(cli) > 0:
+                        return {'filename': "%s/%s" % (self.basedir, included),
+                                'xmltree': xdata,
+                                'xquery': cli}
+                except lxml.etree.XMLSyntaxError:
+                    self.logger.error('Failed to parse %s' % (included))
+        return {}
+
+    def HandleEvent(self, event):
+        """Handle fam events"""
+        filename = event.filename.split('/')[-1]
+        if filename in self.extras:
+            if event.code2str() == 'exists':
+                return False
+        elif filename != self.basefile:
+            return False
+        if event.code2str() == 'endExist':
+            return False
+        self.load_xml()
+        return True
+
+
 class ClientMetadata(object):
     """This object contains client metadata."""
     def __init__(self, client, profile, groups, bundles,
@@ -46,6 +196,7 @@ class ClientMetadata(object):
                 return grp
         return ''
 
+
 class MetadataQuery(object):
     def __init__(self, by_name, get_clients, by_groups, by_profiles, all_groups, all_groups_in_category):
         # resolver is set later
@@ -65,6 +216,7 @@ class MetadataQuery(object):
     def all(self):
         return [self.by_name(name) for name in self.all_clients()]
 
+
 class Metadata(Bcfg2.Server.Plugin.Plugin,
                Bcfg2.Server.Plugin.Metadata,
                Bcfg2.Server.Plugin.Statistics):
@@ -72,6 +224,7 @@ class Metadata(Bcfg2.Server.Plugin.Plugin,
     __version__ = '$Id$'
     __author__ = 'bcfg-dev@mcs.anl.gov'
     name = "Metadata"
+    sort_order = 500
 
     def __init__(self, core, datastore, watch_clients=True):
         Bcfg2.Server.Plugin.Plugin.__init__(self, core, datastore)
@@ -84,11 +237,13 @@ class Metadata(Bcfg2.Server.Plugin.Plugin,
             except:
                 print("Unable to add file monitor for groups.xml or clients.xml")
                 raise Bcfg2.Server.Plugin.PluginInitError
-   
-        self.pseudo_monitor = isinstance(core.fam, Bcfg2.Server.FileMonitor.Pseudo)
+
+        self.clients_xml = XMLMetadataConfig(self, watch_clients, 'clients.xml')
+        self.groups_xml = XMLMetadataConfig(self, watch_clients, 'groups.xml')
         self.states = {}
         if watch_clients:
-            self.states = {"groups.xml":False, "clients.xml":False}
+            self.states = {"groups.xml": False,
+                           "clients.xml": False}
         self.addresses = {}
         self.auth = dict()
         self.clients = {}
@@ -105,14 +260,13 @@ class Metadata(Bcfg2.Server.Plugin.Plugin,
         self.floating = []
         self.passwords = {}
         self.session_cache = {}
-        self.clientdata = None
-        self.clientdata_original = None
         self.default = None
         self.pdirty = False
-        self.extra = {'groups.xml':[], 'clients.xml':[]}
+        self.extra = {'groups.xml': [],
+                      'clients.xml': []}
         self.password = core.password
         self.query = MetadataQuery(core.build_metadata,
-                                   lambda:self.clients.keys(),
+                                   lambda: list(self.clients.keys()),
                                    self.get_client_names_by_groups,
                                    self.get_client_names_by_profiles,
                                    self.get_all_group_names,
@@ -145,72 +299,45 @@ class Metadata(Bcfg2.Server.Plugin.Plugin,
 
     def add_group(self, group_name, attribs):
         """Add group to groups.xml."""
-        tree = lxml.etree.parse(self.data + "/groups.xml")
-        root = tree.getroot()
-        element = lxml.etree.Element("Group", name=group_name)
-        for key, val in attribs.iteritems():
-            element.set(key, val)
-        node = self.search_group(group_name, tree)
+
+        node = self.search_group(group_name, self.groups_xml.xdata)
         if node != None:
             self.logger.error("Group \"%s\" already exists" % (group_name))
             raise MetadataConsistencyError
-        root.append(element)
-        group_tree = open(self.data + "/groups.xml","w")
-        fd = group_tree.fileno()
-        while True:
-            try:
-                fcntl.lockf(fd, fcntl.LOCK_EX | fcntl.LOCK_NB)
-            except IOError:
-                continue
-            else:
-                break
-        tree.write(group_tree)
-        fcntl.lockf(fd, fcntl.LOCK_UN)
-        group_tree.close()
+
+        element = lxml.etree.SubElement(self.groups_xml.base_xdata.getroot(),
+                                      "Group", name=group_name)
+        for key, val in list(attribs.items()):
+            element.set(key, val)
+        self.groups_xml.write()
 
     def update_group(self, group_name, attribs):
         """Update a groups attributes."""
-        tree = lxml.etree.parse(self.data + "/groups.xml")
-        root = tree.getroot()
-        node = self.search_group(group_name, tree)
+        node = self.search_group(group_name, self.groups_xml.xdata)
         if node == None:
-            self.logger.error("Group \"%s\" not found" % (group_name))
+            self.logger.error("Group \"%s\" does not exist" % (group_name))
             raise MetadataConsistencyError
-        node.attrib.update(attribs)
-        group_tree = open(self.data + "/groups.xml","w")
-        fd = group_tree.fileno()
-        while True:
-            try:
-                fcntl.lockf(fd, fcntl.LOCK_EX | fcntl.LOCK_NB)
-            except IOError:
-                continue
-            else:
-                break
-        tree.write(group_tree)
-        fcntl.lockf(fd, fcntl.LOCK_UN)
-        group_tree.close()
+        xdict = self.groups_xml.find_xml_for_xpath('.//Group[@name="%s"]' % (node.get('name')))
+        if not xdict:
+            self.logger.error("Unexpected error finding group")
+            raise MetadataConsistencyError
+
+        for key, val in list(attribs.items()):
+            xdict['xquery'][0].set(key, val)
+        self.groups_xml.write_xml(xdict['filename'], xdict['xmltree'])
 
     def remove_group(self, group_name):
         """Remove a group."""
-        tree = lxml.etree.parse(self.data + "/groups.xml")
-        root = tree.getroot()
-        node = self.search_group(group_name, tree)
+        node = self.search_group(group_name, self.groups_xml.xdata)
         if node == None:
-            self.logger.error("Client \"%s\" not found" % (group_name))
+            self.logger.error("Group \"%s\" does not exist" % (group_name))
             raise MetadataConsistencyError
-        root.remove(node)
-        group_tree = open(self.data + "/groups.xml","w")
-        fd = group_tree.fileno()
-        while True:
-            try:
-                fcntl.lockf(fd, fcntl.LOCK_EX | fcntl.LOCK_NB)
-            except IOError:
-                continue
-            else:
-                break
-        tree.write(group_tree)
-        fcntl.lockf(fd, fcntl.LOCK_UN)
-        group_tree.close()
+        xdict = self.groups_xml.find_xml_for_xpath('.//Group[@name="%s"]' % (node.get('name')))
+        if not xdict:
+            self.logger.error("Unexpected error finding group")
+            raise MetadataConsistencyError
+        xdict['xquery'][0].getparent().remove(xdict['xquery'][0])
+        self.groups_xml.write_xml(xdict['filename'], xdict['xmltree'])
 
     def add_bundle(self, bundle_name):
         """Add bundle to groups.xml."""
@@ -222,7 +349,7 @@ class Metadata(Bcfg2.Server.Plugin.Plugin,
             self.logger.error("Bundle \"%s\" already exists" % (bundle_name))
             raise MetadataConsistencyError
         root.append(element)
-        group_tree = open(self.data + "/groups.xml","w")
+        group_tree = open(self.data + "/groups.xml", "w")
         fd = group_tree.fileno()
         while True:
             try:
@@ -244,7 +371,7 @@ class Metadata(Bcfg2.Server.Plugin.Plugin,
             self.logger.error("Bundle \"%s\" not found" % (bundle_name))
             raise MetadataConsistencyError
         root.remove(node)
-        group_tree = open(self.data + "/groups.xml","w")
+        group_tree = open(self.data + "/groups.xml", "w")
         fd = group_tree.fileno()
         while True:
             try:
@@ -269,83 +396,37 @@ class Metadata(Bcfg2.Server.Plugin.Plugin,
 
     def add_client(self, client_name, attribs):
         """Add client to clients.xml."""
-        tree = lxml.etree.parse(self.data + "/clients.xml")
-        root = tree.getroot()
-        element = lxml.etree.Element("Client", name=client_name)
-        for key, val in attribs.iteritems():
-            element.set(key, val)
-        node = self.search_client(client_name, tree)
+        node = self.search_client(client_name, self.clients_xml.xdata)
         if node != None:
             self.logger.error("Client \"%s\" already exists" % (client_name))
             raise MetadataConsistencyError
-        root.append(element)
-        client_tree = open(self.data + "/clients.xml","w")
-        fd = client_tree.fileno()
-        while True:
-            try:
-                fcntl.lockf(fd, fcntl.LOCK_EX | fcntl.LOCK_NB)
-            except IOError:
-                continue
-            else:
-                break
-        tree.write(client_tree, pretty_print=True)
-        fcntl.lockf(fd, fcntl.LOCK_UN)
-        client_tree.close()
+
+        element = lxml.etree.SubElement(self.clients_xml.base_xdata.getroot(),
+                                      "Client", name=client_name)
+        for key, val in list(attribs.items()):
+            element.set(key, val)
+        self.clients_xml.write()
 
     def update_client(self, client_name, attribs):
         """Update a clients attributes."""
-        tree = lxml.etree.parse(self.data + "/clients.xml")
-        root = tree.getroot()
-        node = self.search_client(client_name, tree)
+        node = self.search_client(client_name, self.clients_xml.xdata)
         if node == None:
-            self.logger.error("Client \"%s\" not found" % (client_name))
+            self.logger.error("Client \"%s\" does not exist" % (client_name))
             raise MetadataConsistencyError
-        node.attrib.update(attribs)
-        client_tree = open(self.data + "/clients.xml","w")
-        fd = client_tree.fileno()
-        while True:
-            try:
-                fcntl.lockf(fd, fcntl.LOCK_EX | fcntl.LOCK_NB)
-            except IOError:
-                continue
-            else:
-                break
-        tree.write(client_tree, pretty_print=True)
-        fcntl.lockf(fd, fcntl.LOCK_UN)
-        client_tree.close()
+
+        xdict = self.clients_xml.find_xml_for_xpath('.//Client[@name="%s"]' % (node.get('name')))
+        if not xdict:
+            self.logger.error("Unexpected error finding client")
+            raise MetadataConsistencyError
+
+        node = xdict['xquery'][0]
+        [node.set(key, value) for key, value in list(attribs.items())]
+        self.clients_xml.write_xml(xdict['filename'], xdict['xmltree'])
 
     def HandleEvent(self, event):
         """Handle update events for data files."""
-        filename = event.filename.split('/')[-1]
-        if filename in ['groups.xml', 'clients.xml']:
-            dest = filename
-        elif filename in reduce(lambda x, y:x+y, self.extra.values()):
-            if event.code2str() == 'exists':
-                return
-            dest = [key for key, value in self.extra.iteritems() if filename in value][0]
-        else:
-            return
-        if event.code2str() == 'endExist':
-            return
-        try:
-            xdata = lxml.etree.parse("%s/%s" % (self.data, dest))
-        except lxml.etree.XMLSyntaxError:
-            self.logger.error('Failed to parse %s' % (dest))
-            return
-        included = [ent.get('href') for ent in \
-                    xdata.findall('./{http://www.w3.org/2001/XInclude}include')]
-        xdata_original = copy.deepcopy(xdata)
-        if included:
-            for name in included:
-                if name not in self.extra[dest]:
-                    self.core.fam.AddMonitor("%s/%s" % (self.data, name), self)
-                    self.extra[dest].append(name)
-            try:
-                xdata.xinclude()
-            except lxml.etree.XIncludeError:
-                self.logger.error("Failed to process XInclude for file %s" % dest)
-
-        if dest == 'clients.xml':
+        if self.clients_xml.HandleEvent(event):
+            xdata = self.clients_xml.xdata
             self.clients = {}
             self.aliases = {}
             self.raliases = {}
@@ -354,8 +435,6 @@ class Metadata(Bcfg2.Server.Plugin.Plugin,
             self.floating = []
             self.addresses = {}
             self.raddresses = {}
-            self.clientdata_original = xdata_original
-            self.clientdata = xdata
             for client in xdata.findall('.//Client'):
                 clname = client.get('name').lower()
                 if 'address' in client.attrib:
@@ -393,15 +472,21 @@ class Metadata(Bcfg2.Server.Plugin.Plugin,
                 self.raliases[clname] = set()
                 [self.raliases[clname].add(alias.get('name')) for alias \
                  in client.findall('Alias')]
-        elif dest == 'groups.xml':
+            self.states['clients.xml'] = True
+        elif self.groups_xml.HandleEvent(event):
+            xdata = self.groups_xml.xdata
             self.public = []
             self.private = []
             self.profiles = []
             self.groups = {}
             grouptmp = {}
             self.categories = {}
-            for group in xdata.xpath('//Groups/Group') \
-                    + xdata.xpath('Group'):
+            groupseen = list()
+            for group in xdata.xpath('//Groups/Group'):
+                if group.get('name') not in groupseen:
+                    groupseen.append(group.get('name'))
+                else:
+                    self.logger.error("Metadata: Group %s defined multiply" % (group.get('name')))
                 grouptmp[group.get('name')] = tuple([[item.get('name') for item in group.findall(spec)]
                                                      for spec in ['./Bundle', './Group']])
                 grouptmp[group.get('name')][1].append(group.get('name'))
@@ -438,18 +523,18 @@ class Metadata(Bcfg2.Server.Plugin.Plugin,
                                                   group_cat[self.categories[ggg]],
                                                   ggg))
                         [self.groups[group][0].add(bund) for bund in bundles]
-        self.states[dest] = True
-        if False not in self.states.values():
+            self.states['groups.xml'] = True
+        if False not in list(self.states.values()):
             # check that all client groups are real and complete
-            real = self.groups.keys()
-            for client in self.clients.keys():
+            real = list(self.groups.keys())
+            for client in list(self.clients.keys()):
                 if self.clients[client] not in self.profiles:
                     self.logger.error("Client %s set as nonexistent or incomplete group %s" \
                                       % (client, self.clients[client]))
                     self.logger.error("Removing client mapping for %s" % (client))
                     self.bad_clients[client] = self.clients[client]
                     del self.clients[client]
-            for bclient in self.bad_clients.keys():
+            for bclient in list(self.bad_clients.keys()):
                 if self.bad_clients[bclient] in self.profiles:
                     self.logger.info("Restored profile mapping for client %s" % bclient)
                     self.clients[bclient] = self.bad_clients[bclient]
@@ -458,85 +543,54 @@ class Metadata(Bcfg2.Server.Plugin.Plugin,
     def set_profile(self, client, profile, addresspair):
         """Set group parameter for provided client."""
         self.logger.info("Asserting client %s profile to %s" % (client, profile))
-        if False in self.states.values():
+        if False in list(self.states.values()):
             raise MetadataRuntimeError
         if profile not in self.public:
             self.logger.error("Failed to set client %s to private group %s" % (client, profile))
             raise MetadataConsistencyError
-        if self.pseudo_monitor:
-            # if we aren't using fam reload the clients.xml file in case its changed
-            try:
-                self.clientdata_original = lxml.etree.parse(self.data + "/clients.xml")
-            except Exception, e:
-                self.logger.error("Metadata: Failed to relad clients.xml. %s" % e)
-                raise MetadataConsistencyError
         if client in self.clients:
             self.logger.info("Changing %s group from %s to %s" % (client, self.clients[client], profile))
-            cli = self.clientdata_original.xpath('.//Client[@name="%s"]' % (client))
-            cli[0].set('profile', profile)
+            xdict = self.clients_xml.find_xml_for_xpath('.//Client[@name="%s"]' % (client))
+            if not xdict:
+                self.logger.error("Metadata: Unable to update profile for client %s.  Use of Xinclude?" % client)
+                raise MetadataConsistencyError
+            xdict['xquery'][0].set('profile', profile)
+            self.clients_xml.write_xml(xdict['filename'], xdict['xmltree'])
         else:
             self.logger.info("Creating new client: %s, profile %s" % \
                              (client, profile))
             if addresspair in self.session_cache:
                 # we are working with a uuid'd client
-                lxml.etree.SubElement(self.clientdata_original.getroot(),
+                lxml.etree.SubElement(self.clients_xml.base_xdata.getroot(),
                                       'Client',
                                       name=self.session_cache[addresspair][1],
                                       uuid=client, profile=profile,
                                       address=addresspair[0])
             else:
-                lxml.etree.SubElement(self.clientdata_original.getroot(),
+                lxml.etree.SubElement(self.clients_xml.base_xdata.getroot(),
                                       'Client', name=client,
                                       profile=profile)
         self.clients[client] = profile
-        self.write_back_clients()
+        self.clients_xml.write()
 
-    def write_back_clients(self):
-        """Write changes to client.xml back to disk."""
-        try:
-            datafile = open("%s/%s" % (self.data, 'clients.xml.new'), 'w')
-        except IOError:
-            self.logger.error("Failed to write clients.xml.new")
-            raise MetadataRuntimeError
-        # prep data
-        dataroot = self.clientdata_original.getroot()
-        newcontents = lxml.etree.tostring(dataroot, pretty_print=True)
-
-        fd = datafile.fileno()
-        while self.locked(fd) == True:
-            pass
-        try:
-            datafile.write(newcontents)
-        except:
-            fcntl.lockf(fd, fcntl.LOCK_UN)
-            self.logger.error("Metadata: Failed to write new clients data to clients.xml.new", exc_info=1)
-            os.unlink("%s/%s" % (self.data, "clients.xml.new"))
-            raise MetadataRuntimeError
-        datafile.close()
-
-        # check if clients.xml is a symlink
-        clientsxml = "%s/%s" % (self.data, 'clients.xml')
-        if os.path.islink(clientsxml):
-            clientsxml = os.readlink(clientsxml)
-
-        try:
-            os.rename("%s/%s" % (self.data, 'clients.xml.new'), clientsxml)
-        except:
-            self.logger.error("Metadata: Failed to rename clients.xml.new")
-            raise MetadataRuntimeError
-
-    def locked(self, fd):
-        try:
-            fcntl.lockf(fd, fcntl.LOCK_EX | fcntl.LOCK_NB)
-        except IOError:
-            return True
-        return False
-
-    def resolve_client(self, addresspair):
+    def resolve_client(self, addresspair, cleanup_cache=False):
         """Lookup address locally or in DNS to get a hostname."""
         if addresspair in self.session_cache:
+            # client _was_ cached, so there can be some expired entries
+            # we need to clean them up to avoid potentially infinite memory swell
+            cache_ttl = 90
+            if cleanup_cache:
+                # remove entries for this client's IP address with _any_ port numbers
+                # - perhaps a priority queue could be faster?
+                curtime = time.time()
+                for addrpair in self.session_cache.keys():
+                     if addresspair[0] == addrpair[0]:
+                         (stamp, _) = self.session_cache[addrpair]
+                         if curtime - stamp > cache_ttl:
+                             del self.session_cache[addrpair]
+            # return the cached data
             (stamp, uuid) = self.session_cache[addresspair]
-            if time.time() - stamp < 90:
+            if time.time() - stamp < cache_ttl:
                 return self.session_cache[addresspair][1]
         address = addresspair[0]
         if address in self.addresses:
@@ -556,7 +610,7 @@ class Metadata(Bcfg2.Server.Plugin.Plugin,
 
     def get_initial_metadata(self, client):
         """Return the metadata for a given client."""
-        if False in self.states.values():
+        if False in list(self.states.values()):
             raise MetadataRuntimeError
         client = client.lower()
         if client in self.aliases:
@@ -581,7 +635,7 @@ class Metadata(Bcfg2.Server.Plugin.Plugin,
             password = self.passwords[client]
         else:
             password = None
-        uuids = [item for item, value in self.uuid.iteritems() if value == client]
+        uuids = [item for item, value in list(self.uuid.items()) if value == client]
         if uuids:
             uuid = uuids[0]
         else:
@@ -599,7 +653,7 @@ class Metadata(Bcfg2.Server.Plugin.Plugin,
 
     def get_all_group_names(self):
         all_groups = set()
-        [all_groups.update(g[1]) for g in self.groups.values()]
+        [all_groups.update(g[1]) for g in list(self.groups.values())]
         return all_groups
 
     def get_all_groups_in_category(self, category):
@@ -609,13 +663,13 @@ class Metadata(Bcfg2.Server.Plugin.Plugin,
         return all_groups
 
     def get_client_names_by_profiles(self, profiles):
-        return [client for client, profile in self.clients.iteritems() \
+        return [client for client, profile in list(self.clients.items()) \
                 if profile in profiles]
 
     def get_client_names_by_groups(self, groups):
-        gprofiles = [profile for profile in self.profiles if \
-                     self.groups[profile][1].issuperset(groups)]
-        return self.get_client_names_by_profiles(gprofiles)
+        mdata = [self.core.build_metadata(client)
+                 for client in list(self.clients.keys())]
+        return [md.hostname for md in mdata if md.groups.issuperset(groups)]
 
     def merge_additional_groups(self, imd, groups):
         for group in groups:
@@ -702,6 +756,9 @@ class Metadata(Bcfg2.Server.Plugin.Plugin,
             return False
 
         if id_method == 'cert' and auth_type != 'cert+password':
+            # remember the cert-derived client name for this connection
+            if client in self.floating:
+                self.session_cache[address] = (time.time(), client)
             # we are done if cert+password not required
             return True
 
@@ -728,7 +785,7 @@ class Metadata(Bcfg2.Server.Plugin.Plugin,
                                   (address[0]))
                 return False
         # populate the session cache
-        if user != 'root':
+        if user.decode('utf-8') != 'root':
             self.session_cache[address] = (time.time(), client)
         return True
 
@@ -737,28 +794,34 @@ class Metadata(Bcfg2.Server.Plugin.Plugin,
         client = meta.hostname
         if client in self.auth and self.auth[client] == 'bootstrap':
             self.logger.info("Asserting client %s auth mode to cert" % client)
-            if self.pseudo_monitor:
-                # if we aren't using fam reload the clients.xml file in case its changed
-                try:
-                    self.clientdata_original = lxml.etree.parse(self.data + "/clients.xml")
-                except Exception, e:
-                    self.logger.error("Metadata: Failed to relad clients.xml. %s" % e)
-                    raise MetadataConsistencyError
-            cli = self.clientdata_original.xpath('.//Client[@name="%s"]' \
-                                                 % (client))
-            cli[0].set('auth', 'cert')
-            self.write_back_clients()
+            xdict = self.clients_xml.find_xml_for_xpath('.//Client[@name="%s"]' % (client))
+            if not xdict:
+                self.logger.error("Metadata: Unable to update profile for client %s.  Use of Xinclude?" % client)
+                raise MetadataConsistencyError
+            xdict['xquery'][0].set('auth', 'cert')
+            self.clients_xml.write_xml(xdict['filename'], xdict['xmltree'])
 
-    def viz(self, hosts, bundles, key, colors):
+    def viz(self, hosts, bundles, key, only_client, colors):
         """Admin mode viz support."""
+        if only_client:
+            clientmeta = self.core.build_metadata(only_client)
+
+        def include_client(client):
+            return not only_client or client != only_client
+
+        def include_bundle(bundle):
+            return not only_client or bundle in clientmeta.bundles
+
+        def include_group(group):
+            return not only_client or group in clientmeta.groups
+        
         groups_tree = lxml.etree.parse(self.data + "/groups.xml")
         try:
             groups_tree.xinclude()
         except lxml.etree.XIncludeError:
             self.logger.error("Failed to process XInclude for file %s" % dest)
         groups = groups_tree.getroot()
-        categories = {'default':'grey83'}
-        instances = {}
+        categories = {'default': 'grey83'}
         viz_str = ""
         egroups = groups.findall("Group") + groups.findall('.//Groups/Group')
         for group in egroups:
@@ -768,13 +831,16 @@ class Metadata(Bcfg2.Server.Plugin.Plugin,
         if None in categories:
             del categories[None]
         if hosts:
+            instances = {}
             clients = self.clients
-            for client, profile in clients.iteritems():
+            for client, profile in list(clients.items()):
+                if include_client(client):
+                    continue
                 if profile in instances:
                     instances[profile].append(client)
                 else:
                     instances[profile] = [client]
-            for profile, clist in instances.iteritems():
+            for profile, clist in list(instances.items()):
                 clist.sort()
                 viz_str += '''\t"%s-instances" [ label="%s", shape="record" ];\n''' \
                     % (profile, '|'.join(clist))
@@ -784,7 +850,8 @@ class Metadata(Bcfg2.Server.Plugin.Plugin,
             bundles = []
             [bundles.append(bund.get('name')) \
                  for bund in groups.findall('.//Bundle') \
-                 if bund.get('name') not in bundles]
+                 if bund.get('name') not in bundles \
+                     and include_bundle(bund.get('name'))]
             bundles.sort()
             for bundle in bundles:
                 viz_str += '''\t"bundle-%s" [ label="%s", shape="septagon"];\n''' \
@@ -796,20 +863,22 @@ class Metadata(Bcfg2.Server.Plugin.Plugin,
             else:
                 style = "filled"
             gseen.append(group.get('name'))
-            viz_str += '\t"group-%s" [label="%s", style="%s", fillcolor=%s];\n' % \
-                (group.get('name'), group.get('name'), style, group.get('color'))
-            if bundles:
-                for bundle in group.findall('Bundle'):
-                    viz_str += '\t"group-%s" -> "bundle-%s";\n' % \
-                        (group.get('name'), bundle.get('name'))
+            if include_group(group.get('name')):
+                viz_str += '\t"group-%s" [label="%s", style="%s", fillcolor=%s];\n' % \
+                           (group.get('name'), group.get('name'), style, group.get('color'))
+                if bundles:
+                    for bundle in group.findall('Bundle'):
+                        viz_str += '\t"group-%s" -> "bundle-%s";\n' % \
+                                   (group.get('name'), bundle.get('name'))
         gfmt = '\t"group-%s" [label="%s", style="filled", fillcolor="grey83"];\n'
         for group in egroups:
             for parent in group.findall('Group'):
-                if parent.get('name') not in gseen:
+                if parent.get('name') not in gseen and include_group(parent.get('name')):
                     viz_str += gfmt % (parent.get('name'), parent.get('name'))
                     gseen.append(parent.get("name"))
-                viz_str += '\t"group-%s" -> "group-%s" ;\n' % \
-                    (group.get('name'), parent.get('name'))
+                if include_group(group.get('name')):
+                    viz_str += '\t"group-%s" -> "group-%s" ;\n' % \
+                               (group.get('name'), parent.get('name'))
         if key:
             for category in categories:
                 viz_str += '''\t"''' + category + '''" [label="''' + category + \
