@@ -1,7 +1,14 @@
+import django
+import sys
+
+# Compatibility import
+from Bcfg2.Bcfg2Py3k import ConfigParser
 # Django settings for bcfg2 reports project.
-from ConfigParser import ConfigParser, NoSectionError, NoOptionError
-c = ConfigParser()
-c.read(['/etc/bcfg2.conf', '/etc/bcfg2-web.conf'])
+c = ConfigParser.ConfigParser()
+if len(c.read(['/etc/bcfg2.conf', '/etc/bcfg2-web.conf'])) == 0:
+    print("Please check that bcfg2.conf or bcfg2-web.conf exists "
+          "and is readable by your web server.")
+    sys.exit(1)
 
 try:
     dset = c.get('statistics', 'web_debug')
@@ -16,38 +23,55 @@ else:
 TEMPLATE_DEBUG = DEBUG
 
 ADMINS = (
-     ('Bcfg2', 'admin@email.address'),
+     ('Root', 'root'),
 )
 
 MANAGERS = ADMINS
-
-DATABASE_ENGINE = c.get('statistics', 'database_engine')
-# 'postgresql', 'mysql', 'sqlite3' or 'ado_mssql'.
+try:
+    db_engine = c.get('statistics', 'database_engine')
+except ConfigParser.NoSectionError:
+    e = sys.exc_info()[1]
+    print("Failed to determine database engine: %s" % e)
+    sys.exit(1)
+db_name = ''
 if c.has_option('statistics', 'database_name'):
-    DATABASE_NAME = c.get('statistics', 'database_name')
-else:
-    DATABASE_NAME = ''
-# Or path to database file if using sqlite3.
-#<repository>/etc/brpt.sqlite is default path
+    db_name = c.get('statistics', 'database_name')
+if db_engine == 'sqlite3' and db_name == '':
+    db_name = "%s/etc/brpt.sqlite" % c.get('server', 'repository')
+
+DATABASES = {
+    'default': {
+        'ENGINE': "django.db.backends.%s" % db_engine,
+        'NAME': db_name
+    }
+}
+
+if db_engine != 'sqlite3':
+    DATABASES['default']['USER'] =  c.get('statistics', 'database_user')
+    DATABASES['default']['PASSWORD'] = c.get('statistics', 'database_password')
+    DATABASES['default']['HOST'] = c.get('statistics', 'database_host')
+    try:
+        DATABASES['default']['PORT'] = c.get('statistics', 'database_port')
+    except: # An empty string tells Django to use the default port.
+        DATABASES['default']['PORT'] = ''
+
+if django.VERSION[0] == 1 and django.VERSION[1] < 2:
+    DATABASE_ENGINE = db_engine
+    DATABASE_NAME = DATABASES['default']['NAME']
+    if DATABASE_ENGINE != 'sqlite3':
+        DATABASE_USER = DATABASES['default']['USER']
+        DATABASE_PASSWORD = DATABASES['default']['PASSWORD']
+        DATABASE_HOST = DATABASES['default']['HOST']
+        DATABASE_PORT = DATABASES['default']['PORT']
 
-if DATABASE_ENGINE != 'sqlite3':
-    DATABASE_USER = c.get('statistics', 'database_user')
-    # Not used with sqlite3.
-    DATABASE_PASSWORD = c.get('statistics', 'database_password')
-    # Not used with sqlite3.
-    DATABASE_HOST = c.get('statistics', 'database_host')
-    # Set to empty string for localhost. Not used with sqlite3.
-    DATABASE_PORT = c.get('statistics', 'database_port')
-    # Set to empty string for default. Not used with sqlite3.
-if DATABASE_ENGINE == 'sqlite3' and DATABASE_NAME == '':
-    DATABASE_NAME = "%s/etc/brpt.sqlite" % c.get('server', 'repository')
 
 # Local time zone for this installation. All choices can be found here:
 # http://docs.djangoproject.com/en/dev/ref/settings/#time-zone
 try:
     TIME_ZONE = c.get('statistics', 'time_zone')
 except:
-    TIME_ZONE = 'America/Chicago'
+    if django.VERSION[0] == 1 and django.VERSION[1] > 2:
+        TIME_ZONE = None
 
 # Language code for this installation. All choices can be found here:
 # http://www.w3.org/TR/REC-html40/struct/dirlang.html#langcodes
@@ -62,7 +86,9 @@ MEDIA_ROOT = ''
 
 # URL that handles the media served from MEDIA_ROOT.
 # Example: "http://media.lawrence.com"
-MEDIA_URL = ''
+MEDIA_URL = '/site_media'
+if c.has_option('statistics', 'web_prefix'):
+    MEDIA_URL = c.get('statistics', 'web_prefix').rstrip('/') + MEDIA_URL
 
 # URL prefix for admin media -- CSS, JavaScript and images. Make sure to use a
 # trailing slash.
@@ -76,7 +102,6 @@ SECRET_KEY = 'eb5+y%oy-qx*2+62vv=gtnnxg1yig_odu0se5$h0hh#pc*lmo7'
 TEMPLATE_LOADERS = (
     'django.template.loaders.filesystem.load_template_source',
     'django.template.loaders.app_directories.load_template_source',
-    'django.template.loaders.eggs.load_template_source',
 )
 
 MIDDLEWARE_CLASSES = (
@@ -109,9 +134,26 @@ TEMPLATE_DIRS = (
     # Put strings here, like "/home/html/django_templates".
     # Always use forward slashes, even on Windows.
     '/usr/share/python-support/python-django/django/contrib/admin/templates/',
-    '/usr/share/bcfg2/Reports/templates'
+    'Bcfg2.Server.Reports.reports'
 )
 
+if django.VERSION[0] == 1 and django.VERSION[1] < 2:
+    TEMPLATE_CONTEXT_PROCESSORS = (
+        'django.core.context_processors.auth',
+        'django.core.context_processors.debug',
+        'django.core.context_processors.i18n',
+        'django.core.context_processors.media',
+        'django.core.context_processors.request'
+    )
+else:
+    TEMPLATE_CONTEXT_PROCESSORS = (
+        'django.contrib.auth.context_processors.auth',
+        'django.core.context_processors.debug',
+        'django.core.context_processors.i18n',
+        'django.core.context_processors.media',
+        'django.core.context_processors.request'
+    )
+
 INSTALLED_APPS = (
     'django.contrib.auth',
     'django.contrib.contenttypes',
